/* Copyright 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef GRID_HH
#define GRID_HH

#include <set>
#include <vector>

namespace SharpConstruct
{
	namespace Brush
	{
		class EditData;
	}

	class Grid
	{
	public:
		class Coordinate
		{
		public:
			Coordinate( const unsigned x, const unsigned y, const unsigned z )
			{
				components_[ 0 ] = x;
				components_[ 1 ] = y;
				components_[ 2 ] = z;
			}

			unsigned operator[]( const unsigned i ) const
			{
				return components_[ i ];
			}
			unsigned& operator[]( const unsigned i )
			{
				return components_[ i ];
			}
			Coordinate Offset( const unsigned i, const int v ) const
			{
				Coordinate r( *this );
				r[ i ] += v;
				return r;
			}
			Coordinate Offset( const int x, const int y, const int z ) const
			{
				Coordinate r( *this );
				r[ 0 ] += x;
				r[ 1 ] += y;
				r[ 2 ] += z;
				return r;
			}
		private:
			unsigned components_[ 3 ];
		};

		typedef std::set< const std::vector< unsigned >* > ActiveCellType;

		void SetDivision( const unsigned );

		const std::vector< unsigned >&
		operator()( const unsigned, const unsigned,
			    const unsigned ) const;
		const std::vector< unsigned >&
		operator()( const Coordinate& ) const;
		std::vector< unsigned >&
		operator()( const Coordinate& );
		std::vector< unsigned >&
		operator()( const unsigned, const unsigned,
			    const unsigned );

		void FindActiveCells( const Brush::EditData& );

		Coordinate FindPoint( const Point3D& ) const;
		//const std::vector< unsigned >& FindPoint( const Optimized::Point3D& ) const;

		bool Valid( const Coordinate& ) const;

		const ActiveCellType& ActiveCells() const;
		ActiveCellType& ActiveCells();
		void ClearActiveCells();

		void UpdateCells();
	private:
		void make_cell_neighbors_active_( const Coordinate c );
		void make_cell_active_( const Coordinate c );
		unsigned convert_range_( const unsigned, const float,
					 const float, const float ) const;
		float cell_to_modelspace_( const unsigned, const float,
					   const float, const unsigned ) const;

		std::vector< std::vector< unsigned > > cells_;
		ActiveCellType active_cells_;

		unsigned division_;
	};
}

#endif
