/* Copyright 2004, 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef MESHHISTORY_H
#define MESHHISTORY_H

#include "Mesh.hh"
#include "MeshData.hh"
#include "Utilities.hh"
#include <deque>
#include <sigc++/sigc++.h>

namespace SharpConstruct
{
	class MeshHistory
	{
	public:
		class PartialMesh
		{
		public:
			PartialMesh()
			: _saved( true ), _vertices( NULL ),
			  _colors( NULL ), _polygons( NULL )
			{}

			~PartialMesh()
			{
				if( _vertices && !_vertices->Used() )
					delete _vertices;

				if( _colors && !_colors->Used() )
					delete _colors;

				if( _polygons && !_polygons->Used() )
					delete _polygons;
			}

			bool& Saved()
			{
				return _saved;
			}
			bool Saved() const
			{
				return _saved;
			}
			std::string& Filename()
			{
				return _filename;
			}

			VertexData* Vertices()
			{
				return _vertices;
			}
			ColorData* Colors()
			{
				return _colors;
			}
			PolygonData* Polygons()
			{
				return _polygons;
			}
			void SetVertices( VertexData* pd )
			{
				if( _vertices )
					_vertices->Release();
				_vertices = pd;
				_vertices->Grab();
			}
			void SetColors( ColorData* pd )
			{
				if( _colors )
					_colors->Release();
				_colors = pd;
				_colors->Grab();
			}
			void SetPolygons( PolygonData* pd )
			{
				if( _polygons )
					_polygons->Release();
				_polygons = pd;
				_polygons->Grab();
			}
			
			VisibleElementData& VisibleElements()
			{
				return _visible_elements;
			}

			BoundingBox& Bounds()
			{
				return bounds_;
			}
		private:
			bool _saved;
			std::string _filename;
			VertexData* _vertices;
			ColorData* _colors;
			PolygonData* _polygons;
			VisibleElementData _visible_elements;
			BoundingBox bounds_;
		};

		~MeshHistory();

		static MeshHistory& Instance();
		
		void AddMesh( bool v, bool c, bool p, bool copy = true );
		void Undo();
		void Redo();
		void Clear();
		PartialMesh& GetPreviousComposite();
		PartialMesh& CurrentComposite();
		Mesh& GetCurrentMesh();
		int Maximum() const;
		void SetMaximum( unsigned );
		bool UndoPossible() const;
		bool RedoPossible() const;
		bool Saved() const;
	
		typedef sigc::signal< void, const bool, const bool, const bool > UpdateSignal;
		sigc::signal< void, bool, bool >& Changed();
		UpdateSignal& SignalEntireSectionChanged();
	private:
		MeshHistory( unsigned );

		void _print_debug_info();

		bool _update_composite();
		void _update_proxy();

		sigc::signal< void, bool, bool > _changed;
		UpdateSignal signal_entire_section_changed_;
		int _maximum, _current_index, _current_maximum_index;
		std::deque< PartialMesh > _meshes;
		Mesh* _current_mesh;
		PartialMesh _current_composite;
		PartialMesh _previous_composite;
	};
}

#endif // MESHHISTORY_H
