/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef MODELPREVIEW_HH
#define MODELPREVIEW_HH

#include "Mesh.hh"
#include "MeshStepables.hh"
#include "ModelDisplay.hh"
#include <gtkmm/alignment.h>
#include <gtkmm/box.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/filechooser.h>
#include <gtkmm/progressbar.h>
#include <memory>

namespace SharpConstruct
{
	class ModelPreview : public Gtk::Alignment
	{
	public:
		ModelPreview( Gtk::FileChooser& );

		void Update();
	private:
		bool step_();
		void update_progress_bar_( const double, const std::string& );
		void on_show_toggled_();

		Gtk::FileChooser& chooser_;
		std::auto_ptr< Create::File > file_;
		std::auto_ptr< FinalizeCreation > finalize_creation_;

		Gtk::VBox padder_;
		Gtk::CheckButton show_;
		Gtk::ProgressBar progress_bar_;
		ModelDisplay model_display_;

		bool cancel_;

		Mesh mesh_;

		// Mesh data
		VertexData vertex_data_;
		ColorData color_data_;
		PolygonData polygon_data_;
		bool mesh_saved_;
		VisibleElementData mesh_visel_;
		BoundingBox mesh_bounds_;
		std::string mesh_filename_;

	};
}

#endif
