/* -*- C++ -*-
 *
 * Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

vec4 Ambient;
vec4 Diffuse;
vec4 Specular;

// Note from TC:
//
// The first time this shader is loaded, it compiles fine. But when switching
// to Normals.glsl, then back to Main.glsl, compilation fails with ATI
// Catalyst 6.2 on WinXP. The error message is:
///
// "ERROR: 0:31: '[' :  array must be redeclared with a size before being indexed with a variable"
//
// GLSLangSpec.Full.1.10.59.pdf says on p.47 that gl_LightSource[] is declared
// with size gl_MaxLights, so the array has a size and it is valid to index it
// with a variable. So this must be a driver bug.
//
// As a workaround, I've changed i to 0 everywhere.

void directionalLight(in int i, in vec3 normal)
{
	float nDotVP;         // normal . light direction
	float nDotHV;         // normal . light half vector
	float pf;             // power factor
	
	nDotVP = max(0.0, dot(normal, normalize(vec3 (gl_LightSource[0].position))));
	nDotHV = max(0.0, dot(normal, vec3 (gl_LightSource[0].halfVector)));
	
	if (nDotVP == 0.0)
	{
		pf = 0.0;
	}
	else
	{
		pf = pow(nDotHV, gl_FrontMaterial.shininess);
		
	}
	Ambient  += gl_LightSource[0].ambient;
	Diffuse  += gl_LightSource[0].diffuse * nDotVP;
	Specular += gl_LightSource[0].specular * pf;
}

vec3 fnormal(void)
{
	//Compute the normal 
	vec3 normal = gl_NormalMatrix * gl_Normal;
	normal = normalize(normal);
	return normal;
}

vec4 flight(in vec3 normal, in vec4 ecPosition, float alphaFade)
{
	vec4 color;
	vec3 ecPosition3;
	vec3 eye;
	
	ecPosition3 = (vec3 (ecPosition)) / ecPosition.w;
	eye = vec3 (0.0, 0.0, 1.0);
	
	// Clear the light intensity accumulators
	Ambient  = vec4 (0.0);
	Diffuse  = vec4 (0.0);
	Specular = vec4 (0.0);
	
	directionalLight(0, normal);
	
	vec4 vcol = gl_Color;
	vcol -= vcol.a;
	
	color = Ambient  * vcol +
		Diffuse  * vcol;
	color += Specular * gl_FrontMaterial.specular;
	color = clamp( color, 0.0, 1.0 );
	return vec4( color.rgb, 1.0 );
}


void main()
{
	vec3  transformedNormal;
	float alphaFade = 1.0;
	
	// Eye-coordinate position of vertex, needed in various calculations
	vec4 ecPosition = gl_ModelViewMatrix * gl_Vertex;
	
	// Do fixed functionality vertex transform
	gl_Position = ftransform();
	transformedNormal = fnormal();

	gl_FrontColor = flight( transformedNormal, ecPosition, alphaFade );
	gl_BackColor = flight( -transformedNormal, ecPosition, alphaFade );
}
