/* Copyright 2006 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "BoundingBox.hh"

#include <iostream>

using SharpConstruct::BoundingBox;
using SharpConstruct::Point3D;

BoundingBox::BoundingBox()
{
	points_.resize( 3 );
}

void BoundingBox::Initialize( const Point3D& p )
{
	points_[ min_ ] = p;
	points_[ max_ ] = p;

	update_();
}

void BoundingBox::Expand( const Point3D& p )
{
	if( p.X() < points_[ min_ ].X() )
		points_[ min_ ].X() = p.X();
	if( p.Y() < points_[ min_ ].Y() )
		points_[ min_ ].Y() = p.Y();
	if( p.Z() < points_[ min_ ].Z() )
		points_[ min_ ].Z() = p.Z();

	if( p.X() > points_[ max_ ].X() )
		points_[ max_ ].X() = p.X();
	if( p.Y() > points_[ max_ ].Y() )
		points_[ max_ ].Y() = p.Y();
	if( p.Z() > points_[ max_ ].Z() )
		points_[ max_ ].Z() = p.Z();

	update_();
}

const Point3D& BoundingBox::Center() const
{
	return points_[ center_ ];
}

float BoundingBox::Diameter() const
{
	return diameter_;
}

void BoundingBox::update_()
{
	points_[ center_ ] = Midpoint( points_[ min_ ], points_[ max_ ] );

	diameter_ = Distance( points_[ min_ ], points_[ max_ ] );
}
